from __future__            import print_function
from elasticsearch_tornado import EsClient
from functools             import partial
from tornado               import gen
from tornado               import ioloop
import sys


def et_cmd(subparsers):
    index_subparser = subparsers.add_parser('index')
    index_subparser.set_defaults(func=index)
    index_subparser.add_argument(
        '--file', '-f',
        default  = '-',
        help     = 'file: default STDIN',
    )


# index
@gen.coroutine
def index_coro(args):
    if args.file == '-':
        data_file = sys.stdin
    else:
        data_file = open(args.file, 'r')

    c = EsClient(host=args.host, port=args.port)
    data = '%s\n' % ''.join(data_file.readlines())
    s = yield gen.Task(c.bulk, data)
    print(s.body.rstrip('\n'))


def index(args):
    m = partial(index_coro, args)
    ioloop.IOLoop.current().run_sync(m)



from __future__            import print_function
from elasticsearch_tornado import EsClient
from functools             import partial
from tornado               import gen
from tornado               import ioloop
import sys


def et_cmd(subparsers):
    search_subparser = subparsers.add_parser('search')
    search_subparser.set_defaults(func=search)
    search_subparser.add_argument(
        '--index', '-i',
        action   = 'append',
        default  = None,
    )
    search_subparser.add_argument(
        '--type', '-t',
        action   = 'append',
        dest     = 'doc_type',
        default  = None,
        help     = 'Doc type',
    )
    search_subparser.add_argument(
        '--analyzer', '-a',
        default  = None,
        help     = 'Analyzer',
    )
    search_subparser.add_argument(
        '--body', '-b',
        default  = None,
        help     = 'Query DSL request body',
    )
    search_subparser.add_argument(
        '--default-opterator', '-o',
        dest     = 'operator',
        default  = None,
    )
    search_subparser.add_argument(
        '--exclude', '-e',
        action   = 'append',
        default  = None,
        help     = 'fields to exclude',
    )
    search_subparser.add_argument(
        '--explain', '-x',
        default  = False,
        action   = 'store_true',
        help     = 'explain query',
    )
    search_subparser.add_argument(
        '--fields', '-f',
        action   = 'append',
        default  = None,
    )
    search_subparser.add_argument(
        '--from',
        dest     = 'start',
        default  = 0,
        type     = int,
        help     = 'Starting offset',
    )
    search_subparser.add_argument(
        '--size', '-s',
        default  = 10,
        type     = int,
        help     = 'Page size',
    )
    search_subparser.add_argument(
        '--query', '-q',
        default  = None,
    )


# search
@gen.coroutine
def search_coro(args):
    c = EsClient(host=args.host, port=args.port)

    doc_type = ','.join(args.doc_type) if args.doc_type else None
    exclude  = ','.join(args.exclude) if args.exclude else None
    explain  = '_explain' if args.explain else None
    fields   = ','.join(args.fields) if args.fields else None
    index    = ','.join(args.index) if args.index else None

    s = yield gen.Task(
        c.search,
        **{
            'params': {
                'default_operator': args.operator,
                'doc_type':        args.doc_type,
                'explain':         explain,
                'from':            args.start,
                'index':           args.index,
                'q':               args.query,
                'size':            args.size,
                '_source_exclude': args.exclude,
                '_source_include': fields,
            }
        }
    )
    print(s.body.rstrip('\n'))


def search(args):
    m = partial(search_coro, args)
    ioloop.IOLoop.current().run_sync(m)



from pkgutil import iter_modules
import argparse
import importlib
import os
import sys


def find_subcommands(subparsers):
    for _, mod_name, is_pkg in iter_modules([os.path.dirname(__file__)]):
        if not is_pkg and mod_name not in [sys.modules, 'cli']:
            mod_fullname = 'elasticsearch_tornado.bin.{}'.format(mod_name)
            module = importlib.import_module(mod_fullname)
            if hasattr(module, 'et_cmd'):
                module.et_cmd(subparsers)


def main():
    parser = argparse.ArgumentParser(
        prog = "et",
        description = "ET: Command line utilities for Elasticsearch",
    )

    parser.add_argument(
        '--host',
        default = os.environ.get('ET_HOST','localhost'),
        help    = 'Elasticsearch server: ET_HOST',
    )

    parser.add_argument(
        '-p', '--port',
        default = os.environ.get('ET_PORT', 9200),
        type    = int,
        help    = 'Elasticsearch server: ET_PORT',
    )

    subparsers = parser.add_subparsers()

    find_subcommands(subparsers)

    args = parser.parse_args()

    if hasattr(args, 'func'):
        args.func(args)



from __future__ import absolute_import

VERSION        = (2, 0, 7)
__version__    = VERSION
__versionstr__ = '.'.join(map(str, VERSION))

from .client   import BaseClient
from .cat      import MeowClient
from .cluster  import ClusterClient
from .indices  import IndicesClient
from .nodes    import NodesClient
from .snapshot import SnapshotClient
from .utils    import serialize


class EsClient(MeowClient, ClusterClient, IndicesClient, NodesClient,
        SnapshotClient):

    def __init__(self, *args, **kwargs):
        super(EsClient, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-
from os.path import join, dirname
from setuptools import setup, find_packages
import sys
import os

VERSION        = (2, 0, 7)
__version__    = VERSION
__versionstr__ = '.'.join(map(str, VERSION))

f = open(join(dirname(__file__), 'README.rst'))
long_description = f.read().strip()
f.close()

install_requires = [
    'tornado>=3.2',
]
tests_require = [
    'nose',
    'coverage',
    'mock',
    'pyaml',
    'nosexcover',
    'tornado>=3.2.2',
]

# use external unittest for 2.6
if sys.version_info[:2] == (2, 6):
    install_requires.append('unittest2')

setup(
    name             = 'elasticsearch_tornado',
    description      = "Tornado HTTP client for Elasticsearch",
    license          = "Apache License, Version 2.0",
    url              = "https://github.com/hodgesds/elasticsearch_tornado",
    long_description = long_description,
    version          = __versionstr__,
    author           = "Daniel Hodges",
    author_email     = "hodges.daniel.scott@gmail.com",
    scripts          = ['bin/et'],
    packages         = find_packages(
        where        = '.',
        exclude      = ('tests*', )
    ),
    classifiers      = [
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],

    install_requires = install_requires,
    test_suite       = 'tests.run_tests.run_all',
    tests_require    = tests_require,
)



import collections
import json

def serialize(obj):
    # first check if it's dictionary like
    if isinstance(obj, collections.Mapping):
        return "%s\n" % json.dumps(obj)
    elif isinstance(obj, collections.Iterable):
        return "\n".join( json.dumps(x) for x in obj ) + '\n'
